@@ -93,6 +93,7 @@
 #include "MagickCore/string_.h"
 #include "MagickCore/string-private.h"
 #include "MagickCore/timer.h"
+#include "MagickCore/timer-private.h"
 #include "MagickCore/token.h"
 #include "MagickCore/utility.h"
 #include "MagickCore/utility-private.h"
@@ -476,6 +477,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   char
     buffer[MagickPathExtent],
     color[MagickPathExtent],
+    iso8601[sizeof("9999-99-99T99:99:99Z")],
     key[MagickPathExtent];
 
   ChannelFeatures
@@ -515,6 +517,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   ImageType
     type;
 
+  int
+    expired;
+
   MagickBooleanType
     ping;
 
@@ -530,6 +535,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   struct stat
     properties;
 
+  struct tm
+    timestamp;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
   if (IsEventLogging() != MagickFalse)
@@ -1662,6 +1670,15 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         image->rows/elapsed_time+0.5),MagickFalse,"P",MagickPathExtent,buffer);
       (void) FormatLocaleFile(file,"  Pixels per second: %s\n",buffer);
     }
+  (void) GetMagickUTCTime(&image->timestamp,&timestamp);
+  (void) strftime(iso8601,sizeof(iso8601),"%FT%TZ",&timestamp);
+  expired=' ';
+  if (IsImageTTLExpired(image) != MagickFalse)
+    expired='*';
+  (void) FormatLocaleFile(file,"  Time-to-live: %g:%g:%g:%g%c %s\n",
+    (double) (image->ttl/(3600*24)),(double) ((image->ttl % (24*3600))/3600),
+    (double) ((image->ttl % 3600)/60),(double) ((image->ttl % 3600) % 60),
+    expired,iso8601);
   (void) FormatLocaleFile(file,"  User time: %0.3fu\n",user_time);
   (void) FormatLocaleFile(file,"  Elapsed time: %lu:%02lu.%03lu\n",
     (unsigned long) (elapsed_time/60.0),(unsigned long) ceil(fmod(elapsed_time,
